@@ -32,7 +32,8 @@
 #include "seahorse-context.h"
 #include "seahorse-widget.h"
 #include "seahorse-secure-memory.h"
-
+#include "seahorse-util.h"
+ 
 /* 
  * Implements tray icon, menu and status window. Tray icon ideas 
  * came from gaim
@@ -153,7 +154,7 @@ window_show ()
     }
 
     if (!g_sctx)
-        g_sctx = seahorse_context_new (TRUE, SKEY_PGP);
+        g_sctx = seahorse_context_new (SEAHORSE_CONTEXT_APP, SKEY_PGP);
     
     g_window = seahorse_widget_new ("agent-cache");
 
@@ -242,7 +243,9 @@ tray_clicked (GtkWidget *button, GdkEventButton *event, void *data)
         glade_xml_signal_connect_data (xml, "on_settings_activate",
                                        G_CALLBACK (on_settings_activate), NULL);
 
-        gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+        gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+                        seahorse_util_determine_popup_menu_position,
+                        (gpointer) button,
                         event->button, gtk_get_current_event_time ());
         gtk_widget_show (menu);
         g_object_unref (xml);
