@@ -193,10 +193,17 @@ prepare_logging ()
     g_log_set_handler ("Gnome", flags, log_handler, NULL);
 }
 
+static void
+client_die ()
+{
+    gtk_main_quit ();
+}
+
 int main(int argc, char* argv[])
 {
     SeahorseOperation *op;
     GnomeProgram *program = NULL;
+    GnomeClient *client = NULL;
 
     seahorse_secure_memory_init (65536);
     
@@ -214,7 +221,7 @@ int main(int argc, char* argv[])
 #endif
         err (1, _("couldn't drop privileges properly"));
     
-    program = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+    program = gnome_program_init("seahorse-daemon", VERSION, LIBGNOMEUI_MODULE, argc, argv,
                     GNOME_PARAM_POPT_TABLE, options,
                     GNOME_PARAM_HUMAN_READABLE_NAME, _("Encryption Daemon (Seahorse)"),
                     GNOME_PARAM_APP_DATADIR, DATA_DIR, NULL);
@@ -234,6 +241,9 @@ int main(int argc, char* argv[])
     signal (SIGTERM, on_quit);
     g_timeout_add (100, check_quit, NULL);
     
+    client = gnome_master_client();
+    g_signal_connect(client, "die", G_CALLBACK(client_die), NULL);
+    
     /* We log to the syslog */
     prepare_logging ();
 
@@ -242,33 +252,42 @@ int main(int argc, char* argv[])
     seahorse_gtkstock_add_icons (daemon_icons);
     
     /* Make the default SeahorseContext */
-    seahorse_context_new (TRUE, 0);
+    seahorse_context_new (SEAHORSE_CONTEXT_APP | SEAHORSE_CONTEXT_DAEMON, 0);
     op = seahorse_context_load_local_keys (SCTX_APP ());
     g_object_unref (op);
     
     /* Initialize the various daemon components */
-#ifdef WITH_DBUS
     seahorse_dbus_server_init ();
-#endif
+
 #ifdef WITH_AGENT
-    seahorse_agent_init ();
-#endif    
+    if (!seahorse_agent_init ())
+        seahorse_agent_uninit ();
+#ifdef WITH_SSH
+    if (!seahorse_agent_ssh_init ())
+        seahorse_agent_ssh_uninit ();
+#endif
+#endif
+    
 #ifdef WITH_SHARING
     seahorse_sharing_init ();
 #endif
 
     gtk_main ();
+g_printerr ("left gtk_main\n");
 
     /* And now clean them all up */
 #ifdef WITH_SHARING
     seahorse_sharing_cleanup ();
 #endif
+    
 #ifdef WITH_AGENT
     seahorse_agent_uninit ();
-#endif    
-#ifdef WITH_DBUS
-    seahorse_dbus_server_cleanup ();
+#ifdef WITH_SSH
+    seahorse_agent_ssh_uninit ();
 #endif
+#endif
+    
+    seahorse_dbus_server_cleanup ();
 
     seahorse_context_destroy (SCTX_APP ());
 
