@@ -27,25 +27,43 @@
 #include "cryptui-keyset.h"
 
 typedef enum {
-    CRYPTUI_ENCTYPE_NONE,
-    CRYPTUI_ENCTYPE_SYMMETRIC,
-    CRYPTUI_ENCTYPE_PUBLIC,
-    CRYPTUI_ENCTYPE_PRIVATE,
+    CRYPTUI_ENCTYPE_NONE =       0,
+    CRYPTUI_ENCTYPE_SYMMETRIC =  1,
+    CRYPTUI_ENCTYPE_PUBLIC =     2,
+    CRYPTUI_ENCTYPE_PRIVATE =    3,
     
     /* Used internally */
     _CRYPTUI_ENCTYPE_MAXVALUE
 } CryptUIEncType;
 
 typedef enum {
-    CRYPTUI_FLAG_IS_VALID = 0x0001,
+    CRYPTUI_FLAG_IS_VALID =    0x0001,
     CRYPTUI_FLAG_CAN_ENCRYPT = 0x0002,
-    CRYPTUI_FLAG_CAN_SIGN = 0x0004,
-    CRYPTUI_FLAG_EXPIRED = 0x0010,
-    CRYPTUI_FLAG_REVOKED = 0x0020,
-    CRYPTUI_FLAG_DISABLED = 0x0040,
-    CRYPTUI_FLAG_TRUSTED = 0x0100
+    CRYPTUI_FLAG_CAN_SIGN =    0x0004,
+    CRYPTUI_FLAG_EXPIRED =     0x0100,
+    CRYPTUI_FLAG_REVOKED =     0x0200,
+    CRYPTUI_FLAG_DISABLED =    0x0400,
+    CRYPTUI_FLAG_TRUSTED =     0x1000
 } CryptUIKeyFlags;
 
+typedef enum {
+    CRYPTUI_LOC_INVALID =        0,    /* An invalid key */
+    CRYPTUI_LOC_MISSING =       10,    /* A key we don't know anything about */
+    CRYPTUI_LOC_SEARCHING =     20,    /* A key we're searching for but haven't found yet */
+    CRYPTUI_LOC_REMOTE =        50,    /* A key that we've found is present remotely */
+    CRYPTUI_LOC_LOCAL =        100,    /* A key on the local machine */
+} CryptUILocation;
+
+typedef enum {
+    CRYPTUI_VALIDITY_REVOKED =   -3,
+    CRYPTUI_VALIDITY_DISABLED =  -2,
+    CRYPTUI_VALIDITY_NEVER =     -1,
+    CRYPTUI_VALIDITY_UNKNOWN =    0,
+    CRYPTUI_VALIDITY_MARGINAL =   1,
+    CRYPTUI_VALIDITY_FULL =       5,
+    CRYPTUI_VALIDITY_ULTIMATE =  10
+} CryptUIValidity;
+
 /* 
  * Key Properties:
  * 
@@ -61,6 +79,9 @@ gchar*              cryptui_key_get_base (const gchar *key);
 
 CryptUIEncType      cryptui_key_get_enctype (const gchar *key);
 
+void                cryptui_display_notification (const gchar *title, const gchar *body,
+                                                  const gchar *icon, gboolean urgent);
+
 gchar**             cryptui_prompt_recipients (CryptUIKeyset *keyset, 
                                                const gchar *title, gchar **signer);
 
