@@ -19,10 +19,18 @@
  * Boston, MA 02111-1307, USA.
  */
  
+#include "config.h"
 #include <gnome.h>
 
 #include "seahorse-key-source.h"
 #include "seahorse-marshal.h"
+#include "seahorse-context.h"
+#include "seahorse-pgp-key.h"
+#include "seahorse-util.h"
+
+#ifdef WITH_SSH
+#include "seahorse-ssh-key.h"
+#endif
 
 G_DEFINE_TYPE (SeahorseKeySource, seahorse_key_source, G_TYPE_OBJECT);
 
@@ -76,7 +84,7 @@ seahorse_key_source_finalize (GObject *gobject)
  **/   
 SeahorseOperation*
 seahorse_key_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad load,
-                          const gchar *match)
+                          GQuark keyid, const gchar *match)
 {
     SeahorseKeySourceClass *klass;
     
@@ -84,7 +92,7 @@ seahorse_key_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad load,
     klass = SEAHORSE_KEY_SOURCE_GET_CLASS (sksrc);
     g_return_val_if_fail (klass->load != NULL, NULL);
     
-    return (*klass->load) (sksrc, load, match);    
+    return (*klass->load) (sksrc, load, keyid, match);
 }
 
 /**
@@ -96,9 +104,9 @@ seahorse_key_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad load,
  **/   
 void
 seahorse_key_source_load_sync (SeahorseKeySource *sksrc, SeahorseKeySourceLoad load,
-                               const gchar *match)
+                               GQuark keyid, const gchar *match)
 {
-    SeahorseOperation *op = seahorse_key_source_load (sksrc, load, match);
+    SeahorseOperation *op = seahorse_key_source_load (sksrc, load, keyid, match);
     g_return_if_fail (op != NULL);
     seahorse_operation_wait (op);
     g_object_unref (op);
@@ -113,9 +121,9 @@ seahorse_key_source_load_sync (SeahorseKeySource *sksrc, SeahorseKeySourceLoad l
  **/   
 void
 seahorse_key_source_load_async (SeahorseKeySource *sksrc, SeahorseKeySourceLoad load,
-                                const gchar *match)
+                                GQuark keyid, const gchar *match)
 {
-    SeahorseOperation *op = seahorse_key_source_load (sksrc, load, match);
+    SeahorseOperation *op = seahorse_key_source_load (sksrc, load, keyid, match);
     g_return_if_fail (op != NULL);
     g_object_unref (op);
 }
@@ -189,17 +197,126 @@ seahorse_key_source_import_sync (SeahorseKeySource *sksrc, gpgme_data_t data,
     return ret;    
 }
 
+SeahorseOperation*
+seahorse_key_source_export_keys (GList *keys, gpgme_data_t data)
+{
+    SeahorseOperation *op = NULL;
+    SeahorseMultiOperation *mop = NULL;
+    SeahorseKeySource *sksrc;
+    SeahorseKey *skey;
+    gboolean allocated = FALSE;
+    GList *next;
+    
+    if (!data) {
+        data = gpgmex_data_new ();
+        allocated = TRUE;
+    }
+    
+    /* Sort by key source */
+    keys = g_list_copy (keys);
+    keys = seahorse_util_keylist_sort (keys);
+    
+    while (keys) {
+     
+        /* Break off one set (same keysource) */
+        next = seahorse_util_keylist_splice (keys);
+
+        g_assert (SEAHORSE_IS_KEY (keys->data));
+        skey = SEAHORSE_KEY (keys->data);
+
+        /* Export from this key source */        
+        sksrc = seahorse_key_get_source (skey);
+        g_return_val_if_fail (sksrc != NULL, FALSE);
+        
+        if (op != NULL) {
+            if (mop == NULL)
+                mop = seahorse_multi_operation_new ();
+            seahorse_multi_operation_take (mop, op);
+        }
+        
+        /* We pass our own data object, to which data is appended */
+        op = seahorse_key_source_export (sksrc, keys, FALSE, data);
+        g_return_val_if_fail (op != NULL, FALSE);
+
+        g_list_free (keys);
+        keys = next;
+    }
+    
+    if (mop) {
+        op = SEAHORSE_OPERATION (mop);
+        
+        /* 
+         * Setup the result data properly, as we would if it was a 
+         * single export operation.
+         */
+        seahorse_operation_mark_result (op, data, 
+                             allocated ? (GDestroyNotify)gpgmex_data_release : NULL);
+    }
+    
+    if (!op) 
+        op = seahorse_operation_new_complete (NULL);
+    
+    return op;
+}
+
 SeahorseOperation* 
 seahorse_key_source_export (SeahorseKeySource *sksrc, GList *keys, 
                             gboolean complete, gpgme_data_t data)
 {
     SeahorseKeySourceClass *klass;
+    SeahorseOperation *op;
+    GSList *keyids = NULL;
+    GList *l;
     
     g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (sksrc), NULL);
     klass = SEAHORSE_KEY_SOURCE_GET_CLASS (sksrc);   
-    g_return_val_if_fail (klass->export != NULL, NULL);
+    if (klass->export) 
+        return (*klass->export) (sksrc, keys, complete, data);    
+
+    /* Either export or export_raw must be implemented */
+    g_return_val_if_fail (klass->export_raw != NULL, NULL);
     
-    return (*klass->export) (sksrc, keys, complete, data);    
+    for (l = keys; l; l = g_list_next (l)) 
+        keyids = g_slist_prepend (keyids, GUINT_TO_POINTER (seahorse_key_get_keyid (l->data)));
+    
+    keyids = g_slist_reverse (keyids);
+    op = (*klass->export_raw) (sksrc, keyids, data);
+    g_slist_free (keyids);
+    return op;
+
+}
+
+SeahorseOperation* 
+seahorse_key_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids, 
+                                gpgme_data_t data)
+{
+    SeahorseKeySourceClass *klass;
+    SeahorseOperation *op;
+    SeahorseKey *skey;
+    GList *keys = NULL;
+    GSList *l;
+    
+    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (sksrc), NULL);
+    klass = SEAHORSE_KEY_SOURCE_GET_CLASS (sksrc);   
+    
+    /* Either export or export_raw must be implemented */
+    if (klass->export_raw)
+        return (*klass->export_raw)(sksrc, keyids, data);
+    
+    g_return_val_if_fail (klass->export != NULL, NULL);
+        
+    for (l = keyids; l; l = g_slist_next (l)) {
+        skey = seahorse_context_get_key (SCTX_APP (), sksrc, GPOINTER_TO_UINT (l->data));
+        
+        /* TODO: A proper error message here 'not found' */
+        if (skey)
+            keys = g_list_prepend (keys, skey);
+    }
+    
+    keys = g_list_reverse (keys);
+    op = (*klass->export) (sksrc, keys, FALSE, data);
+    g_list_free (keys);
+    return op;
 }
 
 gboolean            
@@ -234,3 +351,25 @@ seahorse_key_source_get_location (SeahorseKeySource *sksrc)
     return loc;
 }
 
+/* -----------------------------------------------------------------------------
+ * CANONICAL KEYIDS 
+ */
+
+GQuark
+seahorse_key_source_cannonical_keyid (GQuark ktype, const gchar *keyid)
+{
+    g_return_val_if_fail (keyid != NULL, 0);
+    
+    if (ktype == SKEY_PGP)
+        return seahorse_pgp_key_get_cannonical_id (keyid);
+    
+#ifdef WITH_SSH
+    else if(ktype == SKEY_SSH)
+        return seahorse_ssh_key_get_cannonical_id (keyid);
+#endif
+        
+    else 
+        g_return_val_if_reached (0);
+    
+    return 0;
+}
