@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004-2006 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,17 @@
 #define DEBUG_OPERATION(x) 
 #endif
 
+/* -----------------------------------------------------------------------------
+ * SEAHORSE OPERATION
+ */
+
+enum {
+    PROP_0,
+    PROP_PROGRESS,
+    PROP_MESSAGE,
+    PROP_RESULT
+};
+
 enum {
     DONE,
     PROGRESS,
@@ -50,32 +61,98 @@ enum {
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
-static GObjectClass *parent_class = NULL;
 
-/* GObject handlers */
-static void seahorse_operation_class_init (SeahorseOperationClass *klass);
-static void seahorse_operation_init       (SeahorseOperation *operation);
-static void seahorse_operation_dispose    (GObject *gobject);
-static void seahorse_operation_finalize   (GObject *gobject);
+G_DEFINE_TYPE (SeahorseOperation, seahorse_operation, G_TYPE_OBJECT);
 
-GType
-seahorse_operation_get_type (void)
+/* HELPERS ------------------------------------------------------------------ */
+
+static gboolean
+delayed_mark_done (SeahorseOperation *operation)
 {
-    static GType operation_type = 0;
- 
-    if (!operation_type) {
-        
-        static const GTypeInfo operation_info = {
-            sizeof (SeahorseOperationClass), NULL, NULL,
-            (GClassInitFunc) seahorse_operation_class_init, NULL, NULL,
-            sizeof (SeahorseOperation), 0, (GInstanceInitFunc) seahorse_operation_init
-        };
-        
-        operation_type = g_type_register_static (G_TYPE_OBJECT, 
-                                "SeahorseOperation", &operation_info, 0);
+    g_signal_emit (operation, signals[DONE], 0);
+
+    /* A running operation always refs itself */
+    g_object_unref (operation);   
+    return FALSE;
+}
+
+/* OBJECT ------------------------------------------------------------------- */
+
+static void
+seahorse_operation_init (SeahorseOperation *operation)
+{
+    /* This is the state of non-started operation. all other flags are zero */
+    operation->progress = -1;
+}
+
+static void
+seahorse_operation_set_property (GObject *object, guint prop_id, 
+                                 const GValue *value, GParamSpec *pspec)
+{
+    SeahorseOperation *op = SEAHORSE_OPERATION (object);
+    
+    switch (prop_id) {
+    case PROP_MESSAGE:
+        g_free (op->message);
+        op->message = g_value_dup_string (value);
+        break;
     }
-  
-    return operation_type;
+}
+
+static void
+seahorse_operation_get_property (GObject *object, guint prop_id, 
+                                 GValue *value, GParamSpec *pspec)
+{
+    SeahorseOperation *op = SEAHORSE_OPERATION (object);
+    
+    switch (prop_id) {
+    case PROP_PROGRESS:
+        g_value_set_double (value, op->progress);
+        break;
+    case PROP_MESSAGE:
+        g_value_set_string (value, op->message);
+        break;
+    case PROP_RESULT:
+        g_value_set_pointer (value, op->result);
+        break;
+    }
+}
+
+/* dispose of all our internal references */
+static void
+seahorse_operation_dispose (GObject *gobject)
+{
+    SeahorseOperation *op = SEAHORSE_OPERATION (gobject);
+    
+    if (op->is_running)
+        seahorse_operation_cancel (op);
+    g_assert (!op->is_running);
+
+    /* We do this in dispose in case it's a circular reference */
+    if (op->result && op->result_destroy)
+        (op->result_destroy) (op->result);
+    op->result = NULL;
+    op->result_destroy = NULL;
+    
+    G_OBJECT_CLASS (seahorse_operation_parent_class)->dispose (gobject);
+}
+
+/* free private vars */
+static void
+seahorse_operation_finalize (GObject *gobject)
+{
+    SeahorseOperation *op = SEAHORSE_OPERATION (gobject);
+    g_assert (!op->is_running);
+    
+    if (op->error) {
+        g_error_free (op->error);
+        op->error = NULL;
+    }
+        
+    g_free (op->message);
+    op->message = NULL;
+        
+    G_OBJECT_CLASS (seahorse_operation_parent_class)->finalize (gobject);
 }
 
 static void
@@ -83,12 +160,27 @@ seahorse_operation_class_init (SeahorseOperationClass *klass)
 {
     GObjectClass *gobject_class;
    
-    parent_class = g_type_class_peek_parent (klass);
+    seahorse_operation_parent_class = g_type_class_peek_parent (klass);
     gobject_class = G_OBJECT_CLASS (klass);
     
     gobject_class->dispose = seahorse_operation_dispose;
     gobject_class->finalize = seahorse_operation_finalize;
+    gobject_class->set_property = seahorse_operation_set_property;
+    gobject_class->get_property = seahorse_operation_get_property;
 
+    g_object_class_install_property (gobject_class, PROP_PROGRESS,
+        g_param_spec_double ("progress", "Progress position", "Current progress position (fraction between 0 and 1)", 
+                             0.0, 1.0, 0.0, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_MESSAGE,
+        g_param_spec_string ("message", "Status message", "Current progress message", 
+                             NULL, G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class, PROP_RESULT,
+        g_param_spec_pointer ("result", "Operation Result", "Exact value depends on derived class.", 
+                              G_PARAM_READABLE));
+
+    
     signals[DONE] = g_signal_new ("done", SEAHORSE_TYPE_OPERATION, 
                 G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (SeahorseOperationClass, done),
                 NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
@@ -98,46 +190,7 @@ seahorse_operation_class_init (SeahorseOperationClass *klass)
                 NULL, NULL, seahorse_marshal_VOID__STRING_DOUBLE, G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_DOUBLE);
 }
 
-/* Initialize the object */
-static void
-seahorse_operation_init (SeahorseOperation *operation)
-{
-    /* This is the state of non-started operation */
-    operation->current = -1;
-    operation->total = 0;
-}
-
-/* dispose of all our internal references */
-static void
-seahorse_operation_dispose (GObject *gobject)
-{
-    SeahorseOperation *operation;
-    operation = SEAHORSE_OPERATION (gobject);
-    
-    if (!seahorse_operation_is_done (operation))
-        seahorse_operation_cancel (operation);
-        
-    G_OBJECT_CLASS (parent_class)->dispose (gobject);
-}
-
-/* free private vars */
-static void
-seahorse_operation_finalize (GObject *gobject)
-{
-    SeahorseOperation *operation;
-    operation = SEAHORSE_OPERATION (gobject);
-    g_assert (seahorse_operation_is_done (operation));
-    
-    if (operation->error) {
-        g_error_free (operation->error);
-        operation->error = NULL;
-    }
-        
-    g_free (operation->details);
-    operation->details = NULL;
-        
-    G_OBJECT_CLASS (parent_class)->finalize (gobject);
-}
+/* PUBLIC ------------------------------------------------------------------- */
 
 SeahorseOperation*  
 seahorse_operation_new_complete (GError *err)
@@ -151,146 +204,148 @@ seahorse_operation_new_complete (GError *err)
 }
 
 void
-seahorse_operation_cancel (SeahorseOperation *operation)
+seahorse_operation_cancel (SeahorseOperation *op)
 {
     SeahorseOperationClass *klass;
 
-    g_return_if_fail (SEAHORSE_IS_OPERATION (operation));
-    g_return_if_fail (!seahorse_operation_is_done (operation));
+    g_return_if_fail (SEAHORSE_IS_OPERATION (op));
+    g_return_if_fail (op->is_running);
 
-	g_object_ref (operation);
+    g_object_ref (op);
  
-    klass = SEAHORSE_OPERATION_GET_CLASS (operation);
-    g_return_if_fail (klass->cancel != NULL);
+    klass = SEAHORSE_OPERATION_GET_CLASS (op);
     
-    (*klass->cancel) (operation); 
+    /* A derived operation exists */
+    if (klass->cancel)
+        (*klass->cancel) (op); 
     
-	g_object_unref (operation);
+    /* No derived operation exists */
+    else
+        seahorse_operation_mark_done (op, TRUE, NULL);
+    
+    g_object_unref (op);
 }
 
 void                
-seahorse_operation_steal_error (SeahorseOperation *operation, GError **err)
+seahorse_operation_steal_error (SeahorseOperation *op, GError **err)
 {
     g_return_if_fail (err == NULL || *err == NULL);
     if (err) {
-        *err = operation->error;
-        operation->error = NULL;
+        *err = op->error;
+        op->error = NULL;
     }
 }
 
 void                
-seahorse_operation_copy_error  (SeahorseOperation *operation, GError **err)
+seahorse_operation_copy_error  (SeahorseOperation *op, GError **err)
 {
     g_return_if_fail (err == NULL || *err == NULL);
     if (err) 
-        *err = operation->error ? g_error_copy (operation->error) : NULL;
+        *err = op->error ? g_error_copy (op->error) : NULL;
+}
+
+const GError*       
+seahorse_operation_get_error (SeahorseOperation *op)
+{
+    return op->error;
 }
 
 gpointer
-seahorse_operation_get_result (SeahorseOperation *operation)
+seahorse_operation_get_result (SeahorseOperation *op)
 {
-    return g_object_get_data (G_OBJECT (operation), "result");
+    return op->result;
 }
 
 void                
-seahorse_operation_wait (SeahorseOperation *operation)
+seahorse_operation_wait (SeahorseOperation *op)
 {
-    seahorse_util_wait_until (seahorse_operation_is_done (operation));
+    seahorse_util_wait_until (!seahorse_operation_is_running (op));
 }
 
-gdouble
-seahorse_operation_get_progress (SeahorseOperation *operation)
-{
-    if (operation->total == 0)
-        return -1;
-    return (gdouble)(operation->current) / (gdouble)(operation->total);
-}
+/* METHODS FOR DERIVED CLASSES ---------------------------------------------- */
 
-void                
-seahorse_operation_mark_start (SeahorseOperation *operation)
+void
+seahorse_operation_mark_start (SeahorseOperation *op)
 {
-    g_return_if_fail (SEAHORSE_IS_OPERATION (operation));
+    g_return_if_fail (SEAHORSE_IS_OPERATION (op));
         
     /* A running operation always refs itself */
-    g_object_ref (operation);
-    operation->cancelled = FALSE;
-    operation->details = NULL;
-    operation->current = 0;
-    operation->total = 0;
+    g_object_ref (op);
+    op->is_running = TRUE;
+    op->is_done = FALSE;
+    op->is_cancelled = FALSE;
+    op->progress = -1;
+    
+    g_free (op->message);
+    op->message = NULL;
 }
 
 void 
-seahorse_operation_mark_progress (SeahorseOperation *operation, const gchar *details, 
-                                  gint current, gint total)
+seahorse_operation_mark_progress (SeahorseOperation *op, const gchar *message, 
+                                  gdouble progress)
 {
     gboolean emit = FALSE;
     
-    g_return_if_fail (SEAHORSE_IS_OPERATION (operation));
-    g_return_if_fail (operation->total != -1);
-    g_return_if_fail (total >= 0);
-    g_return_if_fail (current >= 0 && current <= total);
-    
-    /* We reserve 'current == total' for complete operations */
-    if (current == total && total != 0)
-        current = total - 1;
-    
-    if (operation->current != current) {
-        operation->current = current;
+    g_return_if_fail (SEAHORSE_IS_OPERATION (op));
+    g_return_if_fail (op->is_running);
+
+    if (progress != op->progress) {
+        op->progress = progress;
         emit = TRUE;
     }
-    
-    if (operation->total != total) {
-        operation->total = total;
-        emit = TRUE;
-    }
-    
-    if (!seahorse_util_string_equals (operation->details, details)) {
-        g_free (operation->details);
-        operation->details = details ? g_strdup (details) : NULL;
+
+    if (!seahorse_util_string_equals (op->message, message)) {
+        g_free (op->message);
+        op->message = message ? g_strdup (message) : NULL;
         emit = TRUE;
     }
 
     if (emit)    
-        g_signal_emit (G_OBJECT (operation), signals[PROGRESS], 0, operation->details, 
-                       seahorse_operation_get_progress (operation));
-    
-    g_return_if_fail (!seahorse_operation_is_done (operation));
+        g_signal_emit (G_OBJECT (op), signals[PROGRESS], 0, op->message, progress);
 }
 
-static gboolean
-delayed_mark_done (SeahorseOperation *operation)
+void
+seahorse_operation_mark_progress_full (SeahorseOperation *op, const gchar *message,
+                                       gdouble current, gdouble total)
 {
-    g_signal_emit (operation, signals[DONE], 0);
-
-    /* A running operation always refs itself */
-    g_object_unref (operation);   
-    return FALSE;
+    if (current > total)
+        current = total;
+    seahorse_operation_mark_progress (op, message, total <= 0 ? -1 : current / total);
 }
 
+
 void                
-seahorse_operation_mark_done (SeahorseOperation *operation, gboolean cancelled,
+seahorse_operation_mark_done (SeahorseOperation *op, gboolean cancelled,
                               GError *error)
 {
-    g_return_if_fail (SEAHORSE_IS_OPERATION (operation));
-    g_return_if_fail (!seahorse_operation_is_done (operation));
+    g_return_if_fail (SEAHORSE_IS_OPERATION (op));
+    g_return_if_fail (op->is_running);
 
-    /* No details on completed operations */
-    g_free (operation->details);
-    operation->details = NULL;
+    /* No message on completed operations */
+    g_free (op->message);
+    op->message = NULL;
     
-    operation->current = operation->total;
-    operation->cancelled = cancelled;
-    operation->error = error;
+    op->progress = cancelled ? -1 : 1.0;
+    op->is_running = FALSE;
+    op->is_done = TRUE;
+    op->is_cancelled = cancelled;
+    op->error = error;
 
-    g_signal_emit (operation, signals[PROGRESS], 0, NULL, 1.0);
-
-    if (operation->total <= 0)
-        operation->total = 1;
-    operation->current = operation->total;
+    g_signal_emit (op, signals[PROGRESS], 0, NULL, op->progress);
 
     /* Since we're asynchronous we guarantee this by going to back out
        to the loop before marking an operation as done */
-    g_timeout_add (0, (GSourceFunc)delayed_mark_done, operation);
+    g_timeout_add (0, (GSourceFunc)delayed_mark_done, op);
+}
+
+void
+seahorse_operation_mark_result (SeahorseOperation *op, gpointer result,
+                                GDestroyNotify destroy_func)
+{
+    if (op->result && op->result_destroy)
+        (op->result_destroy) (op->result);
+    op->result = result;
+    op->result_destroy = destroy_func;
 }
 
 /* -----------------------------------------------------------------------------
@@ -304,95 +359,9 @@ static void seahorse_multi_operation_dispose    (GObject *gobject);
 static void seahorse_multi_operation_finalize   (GObject *gobject);
 static void seahorse_multi_operation_cancel     (SeahorseOperation *operation);
 
-static GObjectClass *multi_parent_class = NULL;
+G_DEFINE_TYPE (SeahorseMultiOperation, seahorse_multi_operation, SEAHORSE_TYPE_OPERATION);
 
-GType
-seahorse_multi_operation_get_type (void)
-{
-    static GType gtype = 0;
- 
-    if (!gtype) {
-        
-        static const GTypeInfo ginfo = {
-            sizeof (SeahorseMultiOperationClass), NULL, NULL,
-            (GClassInitFunc) seahorse_multi_operation_class_init, NULL, NULL,
-            sizeof (SeahorseMultiOperation), 0, (GInstanceInitFunc) seahorse_multi_operation_init
-        };
-        
-        gtype = g_type_register_static (SEAHORSE_TYPE_OPERATION, "SeahorseMultiOperation", 
-                                        &ginfo, 0);
-    }
-  
-    return gtype;
-}
-
-static void
-seahorse_multi_operation_class_init (SeahorseMultiOperationClass *klass)
-{
-    SeahorseOperationClass *op_class = SEAHORSE_OPERATION_CLASS (klass);
-    GObjectClass *gobject_class;
-
-    multi_parent_class = g_type_class_peek_parent (klass);
-    gobject_class = G_OBJECT_CLASS (klass);
-    
-    op_class->cancel = seahorse_multi_operation_cancel;        
-    gobject_class->dispose = seahorse_multi_operation_dispose;
-    gobject_class->finalize = seahorse_multi_operation_finalize;
-}
-
-/* Initialize the object */
-static void
-seahorse_multi_operation_init (SeahorseMultiOperation *mop)
-{
-    mop->operations = NULL;
-}
-
-/* dispose of all our internal references */
-static void
-seahorse_multi_operation_dispose (GObject *gobject)
-{
-    SeahorseMultiOperation *mop;
-    mop = SEAHORSE_MULTI_OPERATION (gobject);
-    
-    /* Anything remaining, gets released  */
-    mop->operations = seahorse_operation_list_free (mop->operations);
-        
-    G_OBJECT_CLASS (parent_class)->dispose (gobject);
-}
-
-/* free private vars */
-static void
-seahorse_multi_operation_finalize (GObject *gobject)
-{
-    SeahorseMultiOperation *mop;
-    mop = SEAHORSE_MULTI_OPERATION (gobject);
-    
-    g_assert (mop->operations == NULL);
-    
-    G_OBJECT_CLASS (parent_class)->finalize (gobject);
-}
-
-static void 
-seahorse_multi_operation_cancel (SeahorseOperation *operation)
-{
-    SeahorseMultiOperation *mop;
-    
-    g_assert (SEAHORSE_IS_MULTI_OPERATION (operation));
-    mop = SEAHORSE_MULTI_OPERATION (operation);
-
-    seahorse_operation_list_cancel (mop->operations);
-    seahorse_operation_list_purge (mop->operations);    
-    
-    seahorse_operation_mark_done (operation, TRUE, 
-                                  SEAHORSE_OPERATION (mop)->error);
-}
-
-SeahorseMultiOperation*  
-seahorse_multi_operation_new ()
-{
-    SeahorseMultiOperation *mop = g_object_new (SEAHORSE_TYPE_MULTI_OPERATION, NULL); 
-    return mop;
-}
+/* HELPERS ------------------------------------------------------------------ */
 
 static void
 multi_operation_progress (SeahorseOperation *operation, const gchar *message, 
@@ -402,56 +371,55 @@ multi_operation_progress (SeahorseOperation *operation, const gchar *message,
     
     g_assert (SEAHORSE_IS_MULTI_OPERATION (mop));
     g_assert (SEAHORSE_IS_OPERATION (operation));  
-	
+    
     list = mop->operations;
     
     /* One or two operations, simple */
     if (g_slist_length (list) <= 1) {
 
-        DEBUG_OPERATION (("[multi-operation 0x%08X] single progress: %s %d/%d\n", (guint)mop, 
-            seahorse_operation_get_details (operation), operation->current, operation->total));
+        DEBUG_OPERATION (("[multi-operation 0x%08X] single progress: %s %lf\n", (guint)mop, 
+            seahorse_operation_get_message (operation), operation->progress));
         
         seahorse_operation_mark_progress (SEAHORSE_OPERATION (mop), 
-                            seahorse_operation_get_details (operation), 
-                            operation->current, operation->total);
+                            seahorse_operation_get_message (operation), 
+                            operation->progress);
 
 
     /* When more than one operation calculate the fraction */
     } else {
     
         SeahorseOperation *op;
-    	gdouble total = 0;
-	    gdouble current = 0;
+        gdouble total = 0;
+        gdouble current = 0;
+        gdouble progress;
         
-        message = seahorse_operation_get_details (operation);
+        message = seahorse_operation_get_message (operation);
 
-    	/* Get the total progress */
+        /* Get the total progress */
         while (list) {
-		    op = SEAHORSE_OPERATION (list->data);
-    		list = g_slist_next (list);
+            op = SEAHORSE_OPERATION (list->data);
+            list = g_slist_next (list);
 
             if (message && !message[0])
                 message = NULL;
             
-            if (!message && !seahorse_operation_is_done (op))
-                message = seahorse_operation_get_details (op);
-		
-	    	if (!seahorse_operation_is_cancelled (op)) {
-		    	if (op->total == 0) {
-    				total += 1;
-	    			current += (seahorse_operation_is_done (op) ? 1 : 0);
-			    } else {
-		    		total += (op->total >= 0 ? op->total : 0);
-    				current += (op->current >= 0 ? op->current : 0);
-	    		}
-		    }
-        }		
-
-        DEBUG_OPERATION (("[multi-operation 0x%08X] multi progress: %s %d/%d\n", (guint)mop, 
-                    message, (gint)current, (gint)total));
+            if (!message && seahorse_operation_is_running (op))
+                message = seahorse_operation_get_message (op);
+            
+            if (!seahorse_operation_is_cancelled (op)) {
+                total += 1;
+                if (op->progress < 0)
+                    current += (seahorse_operation_is_running (op) ? 0 : 1);
+                else
+                    current += op->progress;
+            }
+        } 
         
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (mop), message,
-	    								  current, total);
+        progress = total ? current / total : -1;
+        DEBUG_OPERATION (("[multi-operation 0x%08X] multi progress: %s %lf\n", (guint)mop, 
+                          message, progress));
+        
+        seahorse_operation_mark_progress (SEAHORSE_OPERATION (mop), message, progress);
     }
 }
 
@@ -471,17 +439,17 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
         seahorse_operation_copy_error (op, &(SEAHORSE_OPERATION (mop)->error));
 
     DEBUG_OPERATION (("[multi-operation 0x%08X] part complete (%d): 0x%08X/%s\n", (guint)mop, 
-                g_slist_length (mop->operations), (guint)op, seahorse_operation_get_details (op)));
+                g_slist_length (mop->operations), (guint)op, seahorse_operation_get_message (op)));
         
     /* Are we done with all of them? */
     for (l = mop->operations; l; l = g_slist_next (l)) {
-        if (!seahorse_operation_is_done (SEAHORSE_OPERATION (l->data))) 
+        if (seahorse_operation_is_running (SEAHORSE_OPERATION (l->data))) 
             done = FALSE;
     }
     
     /* Not done, just update progress. */
     if (!done) {
-   		multi_operation_progress (SEAHORSE_OPERATION (mop), NULL, -1, mop);
+        multi_operation_progress (SEAHORSE_OPERATION (mop), NULL, -1, mop);
         return;
     }
 
@@ -490,21 +458,104 @@ multi_operation_done (SeahorseOperation *op, SeahorseMultiOperation *mop)
     /* Remove all the listeners */
     for (l = mop->operations; l; l = g_slist_next (l)) {
         g_signal_handlers_disconnect_by_func (l->data, multi_operation_done, mop);
-        g_signal_handlers_disconnect_by_func (l->data, multi_operation_done, mop);
+        g_signal_handlers_disconnect_by_func (l->data, multi_operation_progress, mop);
     }
 
     mop->operations = seahorse_operation_list_purge (mop->operations);
     
-    seahorse_operation_mark_done (SEAHORSE_OPERATION (mop), FALSE, 
-                                  SEAHORSE_OPERATION (mop)->error);        
+    if (!seahorse_operation_is_cancelled (SEAHORSE_OPERATION (mop))) 
+        seahorse_operation_mark_done (SEAHORSE_OPERATION (mop), FALSE, 
+                                      SEAHORSE_OPERATION (mop)->error);
 }
 
-void                     
+/* OBJECT ------------------------------------------------------------------- */
+
+static void
+seahorse_multi_operation_init (SeahorseMultiOperation *mop)
+{
+    mop->operations = NULL;
+}
+
+/* dispose of all our internal references */
+static void
+seahorse_multi_operation_dispose (GObject *gobject)
+{
+    SeahorseMultiOperation *mop;
+    GSList *l;
+    
+    mop = SEAHORSE_MULTI_OPERATION (gobject);
+    
+    /* Remove all the listeners */
+    for (l = mop->operations; l; l = g_slist_next (l)) {
+        g_signal_handlers_disconnect_by_func (l->data, multi_operation_done, mop);
+        g_signal_handlers_disconnect_by_func (l->data, multi_operation_progress, mop);
+    }
+    
+    /* Anything remaining, gets released  */
+    mop->operations = seahorse_operation_list_free (mop->operations);
+        
+    G_OBJECT_CLASS (seahorse_multi_operation_parent_class)->dispose (gobject);
+}
+
+/* free private vars */
+static void
+seahorse_multi_operation_finalize (GObject *gobject)
+{
+    SeahorseMultiOperation *mop;
+    mop = SEAHORSE_MULTI_OPERATION (gobject);
+    
+    g_assert (mop->operations == NULL);
+    
+    G_OBJECT_CLASS (seahorse_multi_operation_parent_class)->finalize (gobject);
+}
+
+static void 
+seahorse_multi_operation_cancel (SeahorseOperation *operation)
+{
+    SeahorseMultiOperation *mop;
+    
+    g_assert (SEAHORSE_IS_MULTI_OPERATION (operation));
+    mop = SEAHORSE_MULTI_OPERATION (operation);
+
+    seahorse_operation_list_cancel (mop->operations);
+    mop->operations = seahorse_operation_list_purge (mop->operations);    
+    
+    seahorse_operation_mark_done (operation, TRUE, 
+                                  SEAHORSE_OPERATION (mop)->error);
+}
+
+static void
+seahorse_multi_operation_class_init (SeahorseMultiOperationClass *klass)
+{
+    SeahorseOperationClass *op_class = SEAHORSE_OPERATION_CLASS (klass);
+    GObjectClass *gobject_class;
+
+    seahorse_multi_operation_parent_class = g_type_class_peek_parent (klass);
+    gobject_class = G_OBJECT_CLASS (klass);
+    
+    op_class->cancel = seahorse_multi_operation_cancel;        
+    gobject_class->dispose = seahorse_multi_operation_dispose;
+    gobject_class->finalize = seahorse_multi_operation_finalize;
+}
+
+/* PUBLIC ------------------------------------------------------------------- */
+
+SeahorseMultiOperation*  
+seahorse_multi_operation_new ()
+{
+    SeahorseMultiOperation *mop = g_object_new (SEAHORSE_TYPE_MULTI_OPERATION, NULL); 
+    return mop;
+}
+
+void
 seahorse_multi_operation_take (SeahorseMultiOperation* mop, SeahorseOperation *op)
 {
     g_return_if_fail (SEAHORSE_IS_MULTI_OPERATION (mop));
     g_return_if_fail (SEAHORSE_IS_OPERATION (op));
     
+    /* We should never add an operation to itself */
+    g_return_if_fail (SEAHORSE_OPERATION (op) != SEAHORSE_OPERATION (mop));
+    
     if(mop->operations == NULL) {
         DEBUG_OPERATION (("[multi-operation 0x%08X] start\n", (guint)mop));
         seahorse_operation_mark_start (SEAHORSE_OPERATION (mop));
@@ -521,10 +572,10 @@ seahorse_multi_operation_take (SeahorseMultiOperation* mop, SeahorseOperation *o
 }
 
 /* -----------------------------------------------------------------------------
- * OPERATION LIST
+ * OPERATION LIST FUNCTIONS
  */
 
-GSList*             
+GSList*
 seahorse_operation_list_add (GSList *list, SeahorseOperation *operation)
 {
     /* This assumes the main reference */
@@ -555,7 +606,7 @@ seahorse_operation_list_cancel (GSList *list)
         operation = SEAHORSE_OPERATION (list->data);
         list = g_slist_next (list);
         
-        if (!seahorse_operation_is_done (operation))
+        if (seahorse_operation_is_running (operation))
             seahorse_operation_cancel (operation);
     }
 }
@@ -572,7 +623,7 @@ seahorse_operation_list_purge (GSList *list)
         l = p;
         p = g_slist_next (p);
         
-        if (seahorse_operation_is_done (SEAHORSE_OPERATION (l->data))) {
+        if (!seahorse_operation_is_running (SEAHORSE_OPERATION (l->data))) {
             g_object_unref (G_OBJECT (l->data));
 
             list = g_slist_remove_link (list, l);
@@ -582,10 +633,8 @@ seahorse_operation_list_purge (GSList *list)
     
     return list;
 }
-                                                    
 
-                                                    
-GSList*             
+GSList*
 seahorse_operation_list_free (GSList *list)
 {
     GSList *l;
