@@ -210,50 +210,6 @@ seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
 	gtk_text_buffer_set_text (buffer, string->str, string->len);
 }
 
-/**
- * seahorse_util_write_data_to_file:
- * @path: Path of file to write to
- * @data: Data to write
- * @release: Whether to release @data or not
- *
- * Tries to write @data to the file @path. 
- *
- * Returns: GPG_ERR_NO_ERROR if write is successful,
- * GPGME_File_Error if the file could not be opened
- **/
-gpgme_error_t
-seahorse_util_write_data_to_file (const gchar *path, gpgme_data_t data, 
-                                  gboolean release)
-{
-    gpgme_error_t err = GPG_OK;
-	gpgme_data_t file;
-	gchar *buffer;
-	gint nread;
-   
-    gpgme_data_seek (data, 0, SEEK_SET);
-	
-    file = seahorse_vfs_data_create (path, SEAHORSE_VFS_WRITE, &err);
-    if (file != NULL) {
-    	buffer = g_new0 (gchar, 128);
-    	
-    	while ((nread = gpgme_data_read (data, buffer, 128)) > 0) {
-            if(gpgme_data_write (file, buffer, nread) < 0) {
-                gpg_err_code_t e = gpg_err_code_from_errno (errno);
-                err = GPG_E (e);                
-                break;
-            }
-        }
-
-        g_free (buffer);
-    }
-
-    if (release)
-        gpgme_data_release (data);
-    
-    gpgme_data_release (file);	
-	return err;
-}
-
 /**
  * seahorse_util_write_data_to_text:
  * @data: Data to write
@@ -280,10 +236,11 @@ seahorse_util_write_data_to_text (gpgme_data_t data, gboolean release)
 		string = g_string_append_len (string, buffer, nread);
 
     if (release)
-        gpgme_data_release (data);
+        gpgmex_data_release (data);
     
 	text = string->str;
 	g_string_free (string, FALSE);
+    g_free (buffer);
 	
 	return text;
 }
@@ -361,6 +318,8 @@ seahorse_util_print_fd (int fd, const char* s)
         r = write (fd, s, l);
         
         if (r == -1) {
+            if (errno == EPIPE)
+                return FALSE;
             if (errno != EAGAIN && errno != EINTR) {
                 g_critical ("couldn't write data to socket: %s", strerror (errno));
                 return FALSE;
@@ -411,11 +370,8 @@ seahorse_util_filename_for_keys (GList *keys)
         g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
         skey = SEAHORSE_KEY (keys->data);
         t = seahorse_key_get_simple_name (skey);
-        if (t == NULL) {
-            t = g_strdup (seahorse_key_get_keyid (skey));
-            if (strlen (t) > 8)
-                t[8] = 0;
-        }
+        if (t == NULL)
+            t = g_strdup (_("Key Data"));
     } else {
         t = g_strdup (_("Multiple Keys"));
     }
@@ -800,25 +756,6 @@ seahorse_util_chooser_show_key_files (GtkWidget *dialog)
     gtk_file_filter_add_pattern (filter, "*");    
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);       
 }
-    
-void
-seahorse_util_chooser_show_jpeg_files (GtkWidget *dialog)
-{
-	GtkFileFilter* filter = gtk_file_filter_new ();
-     
-    /* Filter for JPEG files, this is meant to be 
-     * used when searching for images to add as a photoid*/
-     
-    gtk_file_filter_set_name (filter, _("All JPEG files"));
-    gtk_file_filter_add_mime_type (filter, "image/jpeg");
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
-    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
-
-    filter = gtk_file_filter_new ();
-    gtk_file_filter_set_name (filter, _("All files"));
-    gtk_file_filter_add_pattern (filter, "*");    
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
-}   
 
 void
 seahorse_util_chooser_show_archive_files (GtkWidget *dialog)
@@ -937,7 +874,8 @@ gboolean
 seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
 {
 	if (suffix == SEAHORSE_SIG_SUFFIX)
-        return g_str_has_suffix (path, SEAHORSE_EXT_SIG);
+        return g_str_has_suffix (path, SEAHORSE_EXT_SIG) || 
+               g_str_has_suffix (path, SEAHORSE_EXT_ASC);
 	else
         return g_str_has_suffix (path, SEAHORSE_EXT_PGP) ||
                g_str_has_suffix (path, SEAHORSE_EXT_GPG) ||
@@ -965,15 +903,13 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
     const gchar *ext;
     gchar *uri;
     gchar *t;
-	
-	if (suffix == SEAHORSE_CRYPT_SUFFIX)
-        ext = seahorse_gconf_get_boolean (ARMOR_KEY) ? SEAHORSE_EXT_ASC : SEAHORSE_EXT_PGP;
-	else if (suffix == SEAHORSE_ASC_SUFFIX)
-		ext = SEAHORSE_EXT_ASC;
-	else
-		ext = SEAHORSE_EXT_SIG;
-	
-	uri = g_strdup_printf ("%s%s", path, ext);
+    
+    if (suffix == SEAHORSE_CRYPT_SUFFIX)
+        ext = SEAHORSE_EXT_PGP;
+    else
+        ext = SEAHORSE_EXT_SIG;
+    
+    uri = g_strdup_printf ("%s%s", path, ext);
     
     if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
@@ -982,7 +918,7 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
         g_free (t);
 
         seahorse_util_chooser_show_key_files (dialog);
-		gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
+        gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
 
         g_free (uri);                
         uri = NULL;
@@ -1008,8 +944,9 @@ seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
     GtkWidget *dialog;
     gchar *uri;
     gchar *t;
-
-	uri =  g_strndup (path, strlen (path) - 4);
+    
+    g_return_val_if_fail (path != NULL, NULL);
+    uri =  g_strndup (path, strlen (path) - 4);
    
     if (prompt && uri && seahorse_util_uri_exists (uri)) {
             
@@ -1211,6 +1148,36 @@ seahorse_util_string_slist_equal (GSList *l1, GSList *l2)
     return !l1 && !l2;   
 }
 
+/* Callback to determine where a popup menu should be placed */
+void
+seahorse_util_determine_popup_menu_position (GtkMenu *menu, int *x, int *y,
+                                             gboolean *push_in, gpointer  gdata)
+{
+        GtkWidget      *widget;
+        GtkRequisition  requisition;
+        gint            menu_xpos;
+        gint            menu_ypos;
+
+        widget = GTK_WIDGET (gdata);
+
+        gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+
+        gdk_window_get_origin (widget->window, &menu_xpos, &menu_ypos);
+
+        menu_xpos += widget->allocation.x;
+        menu_ypos += widget->allocation.y;
+
+
+        if (menu_ypos > gdk_screen_get_height (gtk_widget_get_screen (widget)) / 2)
+                menu_ypos -= requisition.height;
+        else
+                menu_ypos += widget->allocation.height;
+           
+        *x = menu_xpos;
+        *y = menu_ypos;
+        *push_in = TRUE;
+}            
+
 /* -----------------------------------------------------------------------------
  * DNS-SD Stuff 
  */
